@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 5672 2011-03-26 17:03:25Z rousseau $
+ * $Id: ccid_usb.c 5740 2011-05-12 14:17:17Z rousseau $
  */
 
 #define __CCID_USB__
@@ -129,6 +129,7 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 	{ 0x08C3, 0x0401, 0x5000 },	/* Precise Biometrics Precise 250 MC */
 	{ 0x0B0C, 0x0050, 0x0101 },	/* Todos Argos Mini II */
 	{ 0x0DC3, 0x0900, 0x0200 }, /* Athena IDProtect Key v2 */
+	{ 0x03F0, 0x0036, 0x0124 }, /* HP USB CCID Smartcard Keyboard */
 
 	/* the firmware version is not correct since I do not have received a
 	 * working reader yet */
@@ -320,40 +321,37 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				int interface;
 				int num = 0;
 				const unsigned char *device_descriptor;
+				int readerID = (vendorID << 16) + productID;
 
 #ifdef USE_COMPOSITE_AS_MULTISLOT
 				static int static_interface = 1;
 
+				/* simulate a composite device as when libudev is used */
+				if ((GEMALTOPROXDU == readerID)
+					|| (GEMALTOPROXSU == readerID))
 				{
-					/* simulate a composite device as when libudev is used */
-					int readerID = (vendorID << 16) + productID;
+						/*
+						 * We can't talk to the two CCID interfaces
+						 * at the same time (the reader enters a
+						 * dead lock). So we simulate a multi slot
+						 * reader. By default multi slot readers
+						 * can't use the slots at the same time. See
+						 * TAG_IFD_SLOT_THREAD_SAFE
+						 *
+						 * One side effect is that the two readers
+						 * are seen by pcscd as one reader so the
+						 * interface name is the same for the two.
+						 *
+	* So we have:
+	* 0: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 00
+	* 1: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 01
+	* instead of
+	* 0: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 00
+	* 1: Gemalto Prox-DU [Prox-DU Contactless_09A00795] (09A00795) 01 00
+						 */
 
-					if ((GEMALTOPROXDU == readerID)
-						|| (GEMALTOPROXSU == readerID))
-					{
-							/*
-							 * We can't talk to the two CCID interfaces
-							 * at the same time (the reader enters a
-							 * dead lock). So we simulate a multi slot
-							 * reader. By default multi slot readers
-							 * can't use the slots at the same time. See
-							 * TAG_IFD_SLOT_THREAD_SAFE
-							 *
-							 * One side effect is that the two readers
-							 * are seen by pcscd as one reader so the
-							 * interface name is the same for the two.
-							 *
-		* So we have:
-		* 0: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 00
-		* 1: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 01
-		* instead of
-		* 0: Gemalto Prox-DU [Prox-DU Contact_09A00795] (09A00795) 00 00
-		* 1: Gemalto Prox-DU [Prox-DU Contactless_09A00795] (09A00795) 01 00
-							 */
-
-						/* the CCID interfaces are 1 and 2 */
-						interface_number = static_interface;
-					}
+					/* the CCID interfaces are 1 and 2 */
+					interface_number = static_interface;
 				}
 #endif
 				/* is it already opened? */
@@ -427,12 +425,33 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				}
 
 again:
+#ifdef __APPLE__
+				/* Some early Gemalto Ezio CB+ readers have
+				 * bDeviceClass, bDeviceSubClass and bDeviceProtocol set
+				 * to 0xFF (proprietary) instead of 0x00.
+				 *
+				 * So on Mac OS X the reader configuration is not done
+				 * by the OS/kernel and we do it ourself.
+				 */
+				if (GEMALTO_EZIO_CBP == readerID)
+				{
+					r = libusb_set_configuration(dev_handle, 1);
+					if (r < 0)
+					{
+						(void)libusb_close(dev_handle);
+						DEBUG_CRITICAL4("Can't set configuration on %d/%d: %d",
+							bus_number, device_address, r);
+						continue;
+					}
+				}
+#endif
+
 				r = libusb_get_active_config_descriptor(dev, &config_desc);
 				if (r < 0)
 				{
 					(void)libusb_close(dev_handle);
-					DEBUG_CRITICAL3("Can't get config descriptor on %d/%d",
-						bus_number, device_address);
+					DEBUG_CRITICAL4("Can't get config descriptor on %d/%d: %d",
+						bus_number, device_address, r);
 					continue;
 				}
 
