@@ -1,6 +1,6 @@
 /*
     debug.c: log (or not) messages
-    Copyright (C) 2003-2005   Ludovic Rousseau
+    Copyright (C) 2003-2010   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: debug.c 2974 2008-05-28 18:32:52Z rousseau $
+ * $Id: debug.c 4973 2010-06-01 09:43:29Z rousseau $
  */
 
 
@@ -39,12 +39,14 @@ void log_msg(const int priority, const char *fmt, ...)
 {
 	va_list argptr;
 
+	(void)priority;
+
 	va_start(argptr, fmt);
-	vsnprintf(DebugBuffer, DEBUG_BUF_SIZE, fmt, argptr);
+	(void)vsnprintf(DebugBuffer, DEBUG_BUF_SIZE, fmt, argptr);
 	va_end(argptr);
 
 #ifdef LOG_TO_STDERR
-	fprintf(stderr, "%s\n", DebugBuffer);
+	(void)fprintf(stderr, "%s\n", DebugBuffer);
 #endif
 } /* log_msg */
 
@@ -54,6 +56,8 @@ void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
 	int i;
 	char *c, *debug_buf_end;
 
+	(void)priority;
+
 	debug_buf_end = DebugBuffer + DEBUG_BUF_SIZE - 5;
 
 	strncpy(DebugBuffer, msg, sizeof(DebugBuffer)-1);
@@ -61,15 +65,15 @@ void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
 
 	for (i = 0; (i < len) && (c < debug_buf_end); ++i)
 	{
-		sprintf(c, "%02X ", (unsigned char)buffer[i]);
+		(void)sprintf(c, "%02X ", (unsigned char)buffer[i]);
 		c += strlen(c);
 	}
 
 #ifdef LOG_TO_STDERR
 	if (c >= debug_buf_end)
-		fprintf(stderr, "Debug buffer too short\n");
+		(void)fprintf(stderr, "Debug buffer too short\n");
 
-	fprintf(stderr, "%s\n", DebugBuffer);
+	(void)fprintf(stderr, "%s\n", DebugBuffer);
 #endif
 } /* log_xxd */
 
