@@ -17,7 +17,7 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 5637 2011-02-25 09:03:48Z rousseau $ */
+/* $Id: ifdhandler.c 5727 2011-05-09 13:17:37Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -1459,6 +1459,24 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		RxBuffer[p++] = tmp & 0xFF;	/* value in little endian order */
 		RxBuffer[p++] = (tmp >> 8) & 0xFF;
 
+		/* only if the reader has a display */
+		if (ccid_descriptor -> wLcdLayout)
+		{
+			/* wLcdMaxCharacters */
+			RxBuffer[p++] = PCSCv2_PART10_PROPERTY_wLcdMaxCharacters;	/* tag */
+			RxBuffer[p++] = 2;	/* length */
+			tmp = ccid_descriptor -> wLcdLayout & 0xFF;
+			RxBuffer[p++] = tmp & 0xFF;	/* value in little endian order */
+			RxBuffer[p++] = (tmp >> 8) & 0xFF;
+
+			/* wLcdMaxLines */
+			RxBuffer[p++] = PCSCv2_PART10_PROPERTY_wLcdMaxLines;	/* tag */
+			RxBuffer[p++] = 2;	/* length */
+			tmp = ccid_descriptor -> wLcdLayout >> 8;
+			RxBuffer[p++] = tmp & 0xFF;	/* value in little endian order */
+			RxBuffer[p++] = (tmp >> 8) & 0xFF;
+		}
+
 		/* bTimeOut2 */
 		RxBuffer[p++] = PCSCv2_PART10_PROPERTY_bTimeOut2;
 		RxBuffer[p++] = 1;	/* length */
@@ -1736,18 +1754,31 @@ void init_driver(void)
 		PCSCLITE_HP_DROPDIR, BUNDLE);
 
 	rv = bundleParse(infofile, &plist);
-	if (rv)
-		return;
-
-	/* Log level */
-	rv = LTPBundleFindValueWithKey(&plist, "ifdLogLevel", &values);
 	if (0 == rv)
 	{
-		/* convert from hex or dec or octal */
-		LogLevel = strtoul(list_get_at(values, 0), NULL, 0);
+		/* Log level */
+		rv = LTPBundleFindValueWithKey(&plist, "ifdLogLevel", &values);
+		if (0 == rv)
+		{
+			/* convert from hex or dec or octal */
+			LogLevel = strtoul(list_get_at(values, 0), NULL, 0);
 
-		/* print the log level used */
-		DEBUG_INFO2("LogLevel: 0x%.4X", LogLevel);
+			/* print the log level used */
+			DEBUG_INFO2("LogLevel: 0x%.4X", LogLevel);
+		}
+
+		/* Driver options */
+		rv = LTPBundleFindValueWithKey(&plist, "ifdDriverOptions", &values);
+		if (0 == rv)
+		{
+			/* convert from hex or dec or octal */
+			DriverOptions = strtoul(list_get_at(values, 0), NULL, 0);
+
+			/* print the log level used */
+			DEBUG_INFO2("DriverOptions: 0x%.4X", DriverOptions);
+		}
+
+		bundleRelease(&plist);
 	}
 
 	e = getenv("LIBCCID_ifdLogLevel");
@@ -1760,19 +1791,6 @@ void init_driver(void)
 		DEBUG_INFO2("LogLevel from LIBCCID_ifdLogLevel: 0x%.4X", LogLevel);
 	}
 
-	/* Driver options */
-	rv = LTPBundleFindValueWithKey(&plist, "ifdDriverOptions", &values);
-	if (0 == rv)
-	{
-		/* convert from hex or dec or octal */
-		DriverOptions = strtoul(list_get_at(values, 0), NULL, 0);
-
-		/* print the log level used */
-		DEBUG_INFO2("DriverOptions: 0x%.4X", DriverOptions);
-	}
-
-	bundleRelease(&plist);
-
 	/* get the voltage parameter */
 	switch ((DriverOptions >> 4) & 0x03)
 	{
@@ -1932,7 +1950,7 @@ static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 
 	/* may happen with non ISO cards */
 	if ((0 == f) || (0 == d) || (0 == clock_frequency))
-		return 60;	/* 60 seconds */
+		return 60 * 1000;	/* 60 seconds */
 
 	/* EGT */
 	/* see ch. 6.5.3 Extra Guard Time, page 12 of ISO 7816-3 */
@@ -1977,7 +1995,7 @@ static unsigned int T1_card_timeout(double f, double d, int TC1,
 
 	/* may happen with non ISO cards */
 	if ((0 == f) || (0 == d) || (0 == clock_frequency))
-		return 60;	/* 60 seconds */
+		return 60 * 1000;	/* 60 seconds */
 
 	/* see ch. 6.5.2 Transmission factors F and D, page 12 of ISO 7816-3 */
 	etu = f / d / clock_frequency;
